
import gui.run.RunButton;
import gui.run.RunCheckBox;
import gui.run.RunTextField;
import gui.run.RunSpinnerSlider;

import javax.swing.*;
import java.awt.*;

import sound.OscopePanel;
import sound.Oscillator;
import sound.ulaw.UlawCodec;
import math.fourierTransforms.FFT1d;

/**
 * Created by Robert Distinti.
 * User: default
 * Date: Jul 23, 2005
 * Time: 11 aM
 */
public class MyPlayer extends JDialog {
    private OscopePanel op=new OscopePanel();
    private RunSpinnerSlider slider;
    private RunCheckBox isPsd;
    private double[] originalData=new Oscillator(440, 1000).getSineWave();  // set default data
    private int originalLength=originalData.length;
    private double[] filteredData;
    private double[] displayData;
    private UlawCodec playData;

    private double[] copy(double []a){
        double out[] = new double[a.length];
        for (int i = 0; i <a.length; i++) {
            out[i] = a[i];
        }
        return out;
    }

    private double getDouble(RunTextField src){
        double rtn=Double.valueOf(src.getText()).doubleValue();
        return rtn;
    };

    private double logBase2(int length) {
        return (Math.log(length) / Math.log(2));
    }

    private double[] makePowerOfTwo(double[] in){
        int n = (int)Math.floor(logBase2(in.length));
        double out[] = new double[1 << n];
        for (int i = 0; i < out.length; i++) {
            out[i] = in[i];
        }
        originalLength=out.length;
        return out;
    }

    private void scaleData(double scale_){
        double out[] = new double[originalData.length];
        for (int i = 0; i < originalData.length; i++) {
            out[i] = originalData[i]*scale_;
        }
        originalData=out;
    }

    private void open(){
        UlawCodec ulc = new UlawCodec();
        originalData=copy(ulc.getDoubleArray());
        System.out.println("number of samples "+originalData.length);
        dataChanged();

    }

    private void play(){
        if(playData!=null){
            playData.play();
        }
    }

    private void addNoise(double d[], double amp){
        for (int i = 0; i < originalLength; i++) {
            d[i] = d[i] + amp * (Math.random() - 0.5);
        }
    }

    private void dataChanged(){
        originalLength=originalData.length;
        originalData=makePowerOfTwo(originalData);
        SliderChanged();
    }


    private void SliderChanged(){
        double real[] = new double[originalData.length];
        double imag[] = new double[originalData.length];
        real=copy(originalData);

        // transform
        FFT1d f=new FFT1d();
        f.computeForwardFFT(real, imag);

        // perform scaling
        double r[] = f.getRealData();
        double im[] = f.getImaginaryData();
        for (int i = 0; i < r.length; i++) {
            r[i]/=(double)originalLength;
            im[i]/=(double)originalLength;
        }

        int len=r.length/2;
        int shift=slider.getValue();
        int destIdxL=0;
        int destIdxH=r.length-1;
        int srcIdx=(len-1)*shift/100;
        int cnt=len-srcIdx;
        //
        // cut
        double real2[] = new double[r.length];
        double imag2[] = new double[im.length];
        for(int i=0; i<cnt; i++){
            real2[destIdxH]=real2[destIdxL]=r[srcIdx];
            imag2[destIdxH]=imag2[destIdxL]=im[srcIdx];
            destIdxH--;
            destIdxL++;
            srcIdx++;
        }

        // convert back
        f.computeBackwardFFT(real2, imag2);
        //f.computeBackwardFFT(r, im);
        filteredData=f.getRealData();

        displayChanged();
    }


    private void displayChanged(){
        double out[] = new double[originalLength];
        for (int i = 0; i < originalLength; i++) {
            out[i] = filteredData[i];
        }
        displayData=out;
        if(isPsd.isSelected()){
            double imag[] = new double[filteredData.length];
            out=copy(filteredData);
            FFT1d f=new FFT1d();
            f.computeForwardFFT(out, imag);
            double psd[]= f.getMagnitudeSpectrum() ;
            op.setData(psd);
        }else{
            op.setData(displayData);
        }
        playData=new UlawCodec(displayData);
    }


    public MyPlayer(){
        setModal(false);
        setTitle("Pitch Shifter");
        JPanel jp = new JPanel();
        Container c = getContentPane();
        c.setLayout(new BorderLayout());
        c.add(jp,BorderLayout.NORTH);
        c.add(op,BorderLayout.CENTER);
        jp.add(new RunButton("[Open"){
            public void run(){
                open();
            }
        });
        jp.add(slider=new RunSpinnerSlider(new SpinnerNumberModel(0,0,100,10)){
            public void run(){
                SliderChanged();
            }
        });

        jp.add(new RunButton("[Play"){
            public void run(){
                play();
            }
        });
        jp.add(isPsd=new RunCheckBox("Show PS[D"){
            public void run(){
                displayChanged();
            }
        });

        originalData=new Oscillator(440, 2048).getSineWave();
        scaleData(0.5);
        dataChanged();

        setSize(600, 480);
        setVisible(true);
    }


    public static void main(String[] args) {
        new MyPlayer();
    }
}
