package ECE410_Project3;

import gui.ClosableJFrame;
import gui.run.RunButton;
import gui.run.RunSlider;
import math.Mat1;
import math.fourierTransforms.FFT1d;
import math.fourierTransforms.FFT;
import sound.Oscillator;
import sound.OscopePanel;
import sound.ulaw.UlawCodec;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * Created by IntelliJ IDEA.
 * User: Kevin Macknight ECE410
 * Date: Jul 25, 2005
 * Time: 5:23:19 PM
 */
public class freqShift {
    public static void main(String[] args) {
        testAudioFft();

    }

    static int noise = 50;
    static int shift = 4;
    static double[] r_d;
    UlawCodec ulc;

    public static void testAudioFft() {
        UlawCodec ulc = new UlawCodec();
        Oscillator o = new Oscillator(440, 1 << 15);
        r_d = makeIntegralPowerOfTwo(ulc.getDoubleArray());
        double[] r_dd = makeIntegralPowerOfTwo(ulc.getDoubleArray());

        Mat1.scale(r_dd, 1.0 / r_dd.length);
        FFT1d ff = new FFT1d();
        double[] i_dd = new double[r_dd.length];
        ff.computeForwardFFT(r_dd, i_dd);

        OscopePanel op = new OscopePanel(ff.getMagnitudeSpectrum());
        ClosableJFrame cjf = new ClosableJFrame("psd");
        Container c = cjf.getContentPane();
        c.setLayout(new BorderLayout());
        c.add(op, BorderLayout.CENTER);
        JPanel jp = new JPanel();
        jp.setLayout(new FlowLayout());

        JButton playB = new JButton("Play");
        JButton addB = new JButton("Add Noise");
        JButton shiftB = new JButton("Shift");
        JButton removeB = new JButton("Remove Noise");

        jp.add(new JLabel("Frequency Shift: "));
        jp.add(new RunSlider(2, 10, 4) {
            public void run() {
                shift = getValue();
                System.out.println("shift=" + shift);
            }
        }, BorderLayout.NORTH);
        jp.add(playB, BorderLayout.SOUTH);
        jp.add(shiftB, BorderLayout.SOUTH);
        //jp.add(addB, BorderLayout.SOUTH);
        //jp.add(removeB, BorderLayout.SOUTH);

        playB.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                UlawCodec ulc = new UlawCodec(r_d);
                ulc.play();
            }
        });
        shiftB.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                pitchShift(shift);
            }
        });
        addB.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                double n = shift / 100.0;
                addNoise(r_d, n);
            }
        });
        removeB.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                double noisePowerCutoff = 0.000075;
                Mat1.scale(r_d, 1.0 / r_d.length);
                final FFT1d f = new FFT1d();
                double[] imag = new double[r_d.length];
                f.computeForwardFFT(r_d, imag);
                double psd[] = f.getMagnitudeSpectrum();
                noiseFilter(f, noisePowerCutoff);
                f.computeBackwardFFT(f.getRealData(), f.getImaginaryData());
            }
        });

        c.add(jp, BorderLayout.SOUTH);
        cjf.setSize(600, 600);
        cjf.setVisible(true);
    }

    private static void noiseFilter(FFT1d f, double noisePowerCutoff) {
        double psd[] = f.getMagnitudeSpectrum();
        double r[] = f.getRealData();
        double im[] = f.getImaginaryData();
        for (int i = 0; i < psd.length; i++) {
            if (psd[i] < noisePowerCutoff) {
                r[i] = 0;
                im[i] = 0;
            }
            ;
        }
    }

    private static double[] makeIntegralPowerOfTwo(double[] in) {
        int n = logBase2(in.length);
        double out[] = new double[1 << n];
        for (int i = 0; i < out.length; i++) {
            out[i] = in[i];
        }
        return out;
    }

    private static int logBase2(int length) {
        return (int) (Math.log(length) / Math.log(2));
    }

    private static double[] getDouble(float[] audio) {

        double f[] = new double[audio.length];
        for (int i = 0; i < f.length; i++) {
            f[i] = audio[i];
        }
        return f;
    }

    private static float[] getFloat(double[] audio) {

        float f[] = new float[audio.length];
        for (int i = 0; i < f.length; i++) {
            f[i] = (float) audio[i];
        }
        return f;
    }


    public final static void addNoise(double d[], double amp) {
        for (int i = 0; i < d.length; i++) {
            d[i] = d[i] + amp * (Math.random() - 0.5);
        }
    }

    public final static void pitchShift(int shift) {
        int N = r_d.length;

        int N_on_4 = N / shift;

        //Mat1.scale(r_d, 1.0 / r_d.length);
        final FFT1d f = new FFT1d();
        double[] i_d = new double[r_d.length];
        f.computeForwardFFT(r_d, i_d);

        // shift data down
        for (int i = 0; i < N_on_4; i++) {
            //r_d[i] = r_d[i + N_on_4];
            //i_d[i] = i_d[i + N_on_4];
            r_d[i + N_on_4] = r_d[i];
            i_d[i + N_on_4] = i_d[i];
        }
        for (int i = N_on_4; i < N / 2; i++) {
            r_d[i] = 0;
            i_d[i] = 0;
        }
        // reflect about center, assuming a real signal
        int i, j;
        for (i = 0, j = N - 1; i < N / 2; i++, j--) {
            r_d[j] = r_d[i];
            i_d[j] = i_d[i];
        }

        f.computeBackwardFFT(r_d, i_d);
        Mat1.scale(r_d, 1.0 / r_d.length);       
    }
}
